@@ -242,6 +242,10 @@ impl<'a> Resolver<'a> {
 
     /// Returns a [Mark] for an identifier reference.
     fn mark_for_ref(&self, sym: &JsWord) -> Option<Mark> {
+        self.mark_for_ref_inner(sym, false)
+    }
+
+    fn mark_for_ref_inner(&self, sym: &JsWord, stop_an_fn_scope: bool) -> Option<Mark> {
         if self.config.handle_types && self.in_type {
             let mut mark = self.current.mark;
             let mut scope = Some(&self.current);
@@ -255,6 +259,11 @@ impl<'a> Resolver<'a> {
                     }
                     return Some(mark);
                 }
+
+                if cur.kind == ScopeKind::Fn && stop_an_fn_scope {
+                    return None;
+                }
+
                 if let Some(parent) = &cur.parent {
                     mark = parent.mark;
                 }
@@ -273,6 +282,10 @@ impl<'a> Resolver<'a> {
                 return Some(mark);
             }
 
+            if cur.kind == ScopeKind::Fn && stop_an_fn_scope {
+                return None;
+            }
+
             if let Some(parent) = &cur.parent {
                 mark = parent.mark;
             }
@@ -1462,7 +1475,7 @@ impl Hoister<'_, '_> {
     fn add_pat_id(&mut self, id: &mut Ident) {
         if self.in_catch_body {
             // If we have a binding, it's different variable.
-            if self.resolver.mark_for_ref(&id.sym).is_some()
+            if self.resolver.mark_for_ref_inner(&id.sym, true).is_some()
                 && self.catch_param_decls.contains(&id.sym)
             {
                 return;
@@ -1539,20 +1552,16 @@ impl VisitMut for Hoister<'_, '_> {
         let old_exclude = self.excluded_from_catch.clone();
         self.excluded_from_catch = Default::default();
 
+        let old_in_catch_body = self.in_catch_body;
+
         let params: Vec<Id> = find_ids(&c.param);
 
         self.catch_param_decls
             .extend(params.into_iter().map(|v| v.0));
 
-        {
-            let old_in_catch_body = self.in_catch_body;
+        self.in_catch_body = true;
+        c.body.visit_mut_with(self);
 
-            self.in_catch_body = true;
-
-            c.body.visit_mut_with(self);
-
-            self.in_catch_body = old_in_catch_body;
-        }
         let orig = self.catch_param_decls.clone();
 
         // let mut excluded = find_ids::<_, Id>(&c.body);
@@ -1564,10 +1573,12 @@ impl VisitMut for Hoister<'_, '_> {
         //     self.resolver.mark_for_ref(&id.0).is_none()
         // });
 
+        self.in_catch_body = false;
         c.param.visit_mut_with(self);
 
         self.catch_param_decls = orig;
 
+        self.in_catch_body = old_in_catch_body;
         self.excluded_from_catch = old_exclude;
     }
 
